/*******************************************************************************
* File Name: UARTDEBUGINT.c
* Version 2.50
*
* Description:
*  This file provides all Interrupt Service functionality of the UART component
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "UARTDEBUG.h"
#include "cyapicallbacks.h"


/***************************************
* Custom Declarations
***************************************/
/* `#START CUSTOM_DECLARATIONS` Place your declaration here */

/* `#END` */

#if (UARTDEBUG_RX_INTERRUPT_ENABLED && (UARTDEBUG_RX_ENABLED || UARTDEBUG_HD_ENABLED))
    /*******************************************************************************
    * Function Name: UARTDEBUG_RXISR
    ********************************************************************************
    *
    * Summary:
    *  Interrupt Service Routine for RX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  UARTDEBUG_rxBuffer - RAM buffer pointer for save received data.
    *  UARTDEBUG_rxBufferWrite - cyclic index for write to rxBuffer,
    *     increments after each byte saved to buffer.
    *  UARTDEBUG_rxBufferRead - cyclic index for read from rxBuffer,
    *     checked to detect overflow condition.
    *  UARTDEBUG_rxBufferOverflow - software overflow flag. Set to one
    *     when UARTDEBUG_rxBufferWrite index overtakes
    *     UARTDEBUG_rxBufferRead index.
    *  UARTDEBUG_rxBufferLoopDetect - additional variable to detect overflow.
    *     Set to one when UARTDEBUG_rxBufferWrite is equal to
    *    UARTDEBUG_rxBufferRead
    *  UARTDEBUG_rxAddressMode - this variable contains the Address mode,
    *     selected in customizer or set by UART_SetRxAddressMode() API.
    *  UARTDEBUG_rxAddressDetected - set to 1 when correct address received,
    *     and analysed to store following addressed data bytes to the buffer.
    *     When not correct address received, set to 0 to skip following data bytes.
    *
    *******************************************************************************/
    CY_ISR(UARTDEBUG_RXISR)
    {
        uint8 readData;
        uint8 readStatus;
        uint8 increment_pointer = 0u;

    #if(CY_PSOC3)
        uint8 int_en;
    #endif /* (CY_PSOC3) */

    #ifdef UARTDEBUG_RXISR_ENTRY_CALLBACK
        UARTDEBUG_RXISR_EntryCallback();
    #endif /* UARTDEBUG_RXISR_ENTRY_CALLBACK */

        /* User code required at start of ISR */
        /* `#START UARTDEBUG_RXISR_START` */

        /* `#END` */

    #if(CY_PSOC3)   /* Make sure nested interrupt is enabled */
        int_en = EA;
        CyGlobalIntEnable;
    #endif /* (CY_PSOC3) */

        do
        {
            /* Read receiver status register */
            readStatus = UARTDEBUG_RXSTATUS_REG;
            /* Copy the same status to readData variable for backward compatibility support 
            *  of the user code in UARTDEBUG_RXISR_ERROR` section. 
            */
            readData = readStatus;

            if((readStatus & (UARTDEBUG_RX_STS_BREAK | 
                            UARTDEBUG_RX_STS_PAR_ERROR |
                            UARTDEBUG_RX_STS_STOP_ERROR | 
                            UARTDEBUG_RX_STS_OVERRUN)) != 0u)
            {
                /* ERROR handling. */
                UARTDEBUG_errorStatus |= readStatus & ( UARTDEBUG_RX_STS_BREAK | 
                                                            UARTDEBUG_RX_STS_PAR_ERROR | 
                                                            UARTDEBUG_RX_STS_STOP_ERROR | 
                                                            UARTDEBUG_RX_STS_OVERRUN);
                /* `#START UARTDEBUG_RXISR_ERROR` */

                /* `#END` */
                
            #ifdef UARTDEBUG_RXISR_ERROR_CALLBACK
                UARTDEBUG_RXISR_ERROR_Callback();
            #endif /* UARTDEBUG_RXISR_ERROR_CALLBACK */
            }
            
            if((readStatus & UARTDEBUG_RX_STS_FIFO_NOTEMPTY) != 0u)
            {
                /* Read data from the RX data register */
                readData = UARTDEBUG_RXDATA_REG;
            #if (UARTDEBUG_RXHW_ADDRESS_ENABLED)
                if(UARTDEBUG_rxAddressMode == (uint8)UARTDEBUG__B_UART__AM_SW_DETECT_TO_BUFFER)
                {
                    if((readStatus & UARTDEBUG_RX_STS_MRKSPC) != 0u)
                    {
                        if ((readStatus & UARTDEBUG_RX_STS_ADDR_MATCH) != 0u)
                        {
                            UARTDEBUG_rxAddressDetected = 1u;
                        }
                        else
                        {
                            UARTDEBUG_rxAddressDetected = 0u;
                        }
                    }
                    if(UARTDEBUG_rxAddressDetected != 0u)
                    {   /* Store only addressed data */
                        UARTDEBUG_rxBuffer[UARTDEBUG_rxBufferWrite] = readData;
                        increment_pointer = 1u;
                    }
                }
                else /* Without software addressing */
                {
                    UARTDEBUG_rxBuffer[UARTDEBUG_rxBufferWrite] = readData;
                    increment_pointer = 1u;
                }
            #else  /* Without addressing */
                UARTDEBUG_rxBuffer[UARTDEBUG_rxBufferWrite] = readData;
                increment_pointer = 1u;
            #endif /* (UARTDEBUG_RXHW_ADDRESS_ENABLED) */

                /* Do not increment buffer pointer when skip not addressed data */
                if(increment_pointer != 0u)
                {
                    if(UARTDEBUG_rxBufferLoopDetect != 0u)
                    {   /* Set Software Buffer status Overflow */
                        UARTDEBUG_rxBufferOverflow = 1u;
                    }
                    /* Set next pointer. */
                    UARTDEBUG_rxBufferWrite++;

                    /* Check pointer for a loop condition */
                    if(UARTDEBUG_rxBufferWrite >= UARTDEBUG_RX_BUFFER_SIZE)
                    {
                        UARTDEBUG_rxBufferWrite = 0u;
                    }

                    /* Detect pre-overload condition and set flag */
                    if(UARTDEBUG_rxBufferWrite == UARTDEBUG_rxBufferRead)
                    {
                        UARTDEBUG_rxBufferLoopDetect = 1u;
                        /* When Hardware Flow Control selected */
                        #if (UARTDEBUG_FLOW_CONTROL != 0u)
                            /* Disable RX interrupt mask, it is enabled when user read data from the buffer using APIs */
                            UARTDEBUG_RXSTATUS_MASK_REG  &= (uint8)~UARTDEBUG_RX_STS_FIFO_NOTEMPTY;
                            CyIntClearPending(UARTDEBUG_RX_VECT_NUM);
                            break; /* Break the reading of the FIFO loop, leave the data there for generating RTS signal */
                        #endif /* (UARTDEBUG_FLOW_CONTROL != 0u) */
                    }
                }
            }
        }while((readStatus & UARTDEBUG_RX_STS_FIFO_NOTEMPTY) != 0u);

        /* User code required at end of ISR (Optional) */
        /* `#START UARTDEBUG_RXISR_END` */

        /* `#END` */

    #ifdef UARTDEBUG_RXISR_EXIT_CALLBACK
        UARTDEBUG_RXISR_ExitCallback();
    #endif /* UARTDEBUG_RXISR_EXIT_CALLBACK */

    #if(CY_PSOC3)
        EA = int_en;
    #endif /* (CY_PSOC3) */
    }
    
#endif /* (UARTDEBUG_RX_INTERRUPT_ENABLED && (UARTDEBUG_RX_ENABLED || UARTDEBUG_HD_ENABLED)) */


#if (UARTDEBUG_TX_INTERRUPT_ENABLED && UARTDEBUG_TX_ENABLED)
    /*******************************************************************************
    * Function Name: UARTDEBUG_TXISR
    ********************************************************************************
    *
    * Summary:
    * Interrupt Service Routine for the TX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  UARTDEBUG_txBuffer - RAM buffer pointer for transmit data from.
    *  UARTDEBUG_txBufferRead - cyclic index for read and transmit data
    *     from txBuffer, increments after each transmitted byte.
    *  UARTDEBUG_rxBufferWrite - cyclic index for write to txBuffer,
    *     checked to detect available for transmission bytes.
    *
    *******************************************************************************/
    CY_ISR(UARTDEBUG_TXISR)
    {
    #if(CY_PSOC3)
        uint8 int_en;
    #endif /* (CY_PSOC3) */

    #ifdef UARTDEBUG_TXISR_ENTRY_CALLBACK
        UARTDEBUG_TXISR_EntryCallback();
    #endif /* UARTDEBUG_TXISR_ENTRY_CALLBACK */

        /* User code required at start of ISR */
        /* `#START UARTDEBUG_TXISR_START` */

        /* `#END` */

    #if(CY_PSOC3)   /* Make sure nested interrupt is enabled */
        int_en = EA;
        CyGlobalIntEnable;
    #endif /* (CY_PSOC3) */

        while((UARTDEBUG_txBufferRead != UARTDEBUG_txBufferWrite) &&
             ((UARTDEBUG_TXSTATUS_REG & UARTDEBUG_TX_STS_FIFO_FULL) == 0u))
        {
            /* Check pointer wrap around */
            if(UARTDEBUG_txBufferRead >= UARTDEBUG_TX_BUFFER_SIZE)
            {
                UARTDEBUG_txBufferRead = 0u;
            }

            UARTDEBUG_TXDATA_REG = UARTDEBUG_txBuffer[UARTDEBUG_txBufferRead];

            /* Set next pointer */
            UARTDEBUG_txBufferRead++;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START UARTDEBUG_TXISR_END` */

        /* `#END` */

    #ifdef UARTDEBUG_TXISR_EXIT_CALLBACK
        UARTDEBUG_TXISR_ExitCallback();
    #endif /* UARTDEBUG_TXISR_EXIT_CALLBACK */

    #if(CY_PSOC3)
        EA = int_en;
    #endif /* (CY_PSOC3) */
   }
#endif /* (UARTDEBUG_TX_INTERRUPT_ENABLED && UARTDEBUG_TX_ENABLED) */


/* [] END OF FILE */
