/*******************************************************************************
* File Name: UARTDEBUG_PM.c
* Version 2.50
*
* Description:
*  This file provides Sleep/WakeUp APIs functionality.
*
* Note:
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "UARTDEBUG.h"


/***************************************
* Local data allocation
***************************************/

static UARTDEBUG_BACKUP_STRUCT  UARTDEBUG_backup =
{
    /* enableState - disabled */
    0u,
};



/*******************************************************************************
* Function Name: UARTDEBUG_SaveConfig
********************************************************************************
*
* Summary:
*  This function saves the component nonretention control register.
*  Does not save the FIFO which is a set of nonretention registers.
*  This function is called by the UARTDEBUG_Sleep() function.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  UARTDEBUG_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void UARTDEBUG_SaveConfig(void)
{
    #if(UARTDEBUG_CONTROL_REG_REMOVED == 0u)
        UARTDEBUG_backup.cr = UARTDEBUG_CONTROL_REG;
    #endif /* End UARTDEBUG_CONTROL_REG_REMOVED */
}


/*******************************************************************************
* Function Name: UARTDEBUG_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the nonretention control register except FIFO.
*  Does not restore the FIFO which is a set of nonretention registers.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  UARTDEBUG_backup - used when non-retention registers are restored.
*
* Reentrant:
*  No.
*
* Notes:
*  If this function is called without calling UARTDEBUG_SaveConfig() 
*  first, the data loaded may be incorrect.
*
*******************************************************************************/
void UARTDEBUG_RestoreConfig(void)
{
    #if(UARTDEBUG_CONTROL_REG_REMOVED == 0u)
        UARTDEBUG_CONTROL_REG = UARTDEBUG_backup.cr;
    #endif /* End UARTDEBUG_CONTROL_REG_REMOVED */
}


/*******************************************************************************
* Function Name: UARTDEBUG_Sleep
********************************************************************************
*
* Summary:
*  This is the preferred API to prepare the component for sleep. 
*  The UARTDEBUG_Sleep() API saves the current component state. Then it
*  calls the UARTDEBUG_Stop() function and calls 
*  UARTDEBUG_SaveConfig() to save the hardware configuration.
*  Call the UARTDEBUG_Sleep() function before calling the CyPmSleep() 
*  or the CyPmHibernate() function. 
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  UARTDEBUG_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void UARTDEBUG_Sleep(void)
{
    #if(UARTDEBUG_RX_ENABLED || UARTDEBUG_HD_ENABLED)
        if((UARTDEBUG_RXSTATUS_ACTL_REG  & UARTDEBUG_INT_ENABLE) != 0u)
        {
            UARTDEBUG_backup.enableState = 1u;
        }
        else
        {
            UARTDEBUG_backup.enableState = 0u;
        }
    #else
        if((UARTDEBUG_TXSTATUS_ACTL_REG  & UARTDEBUG_INT_ENABLE) !=0u)
        {
            UARTDEBUG_backup.enableState = 1u;
        }
        else
        {
            UARTDEBUG_backup.enableState = 0u;
        }
    #endif /* End UARTDEBUG_RX_ENABLED || UARTDEBUG_HD_ENABLED*/

    UARTDEBUG_Stop();
    UARTDEBUG_SaveConfig();
}


/*******************************************************************************
* Function Name: UARTDEBUG_Wakeup
********************************************************************************
*
* Summary:
*  This is the preferred API to restore the component to the state when 
*  UARTDEBUG_Sleep() was called. The UARTDEBUG_Wakeup() function
*  calls the UARTDEBUG_RestoreConfig() function to restore the 
*  configuration. If the component was enabled before the 
*  UARTDEBUG_Sleep() function was called, the UARTDEBUG_Wakeup()
*  function will also re-enable the component.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  UARTDEBUG_backup - used when non-retention registers are restored.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void UARTDEBUG_Wakeup(void)
{
    UARTDEBUG_RestoreConfig();
    #if( (UARTDEBUG_RX_ENABLED) || (UARTDEBUG_HD_ENABLED) )
        UARTDEBUG_ClearRxBuffer();
    #endif /* End (UARTDEBUG_RX_ENABLED) || (UARTDEBUG_HD_ENABLED) */
    #if(UARTDEBUG_TX_ENABLED || UARTDEBUG_HD_ENABLED)
        UARTDEBUG_ClearTxBuffer();
    #endif /* End UARTDEBUG_TX_ENABLED || UARTDEBUG_HD_ENABLED */

    if(UARTDEBUG_backup.enableState != 0u)
    {
        UARTDEBUG_Enable();
    }
}


/* [] END OF FILE */
