/*******************************************************************************
* File Name: W5200_SPI.h
* Version 2.50
*
* Description:
*  Contains the function prototypes, constants and register definition
*  of the SPI Master Component.
*
* Note:
*  None
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(CY_SPIM_W5200_SPI_H)
#define CY_SPIM_W5200_SPI_H

#include "cyfitter.h"
#include "cytypes.h"
#include "CyLib.h" /* For CyEnterCriticalSection() and CyExitCriticalSection() functions */


/***************************************
*   Conditional Compilation Parameters
***************************************/

#define W5200_SPI_INTERNAL_CLOCK             (1u)

#if(0u != W5200_SPI_INTERNAL_CLOCK)
    #include "W5200_SPI_IntClock.h"
#endif /* (0u != W5200_SPI_INTERNAL_CLOCK) */

#define W5200_SPI_MODE                       (4u)
#define W5200_SPI_DATA_WIDTH                 (8u)
#define W5200_SPI_MODE_USE_ZERO              (0u)
#define W5200_SPI_BIDIRECTIONAL_MODE         (0u)

/* Internal interrupt handling */
#define W5200_SPI_TX_BUFFER_SIZE             (4u)
#define W5200_SPI_RX_BUFFER_SIZE             (4u)
#define W5200_SPI_INTERNAL_TX_INT_ENABLED    (0u)
#define W5200_SPI_INTERNAL_RX_INT_ENABLED    (0u)

#define W5200_SPI_SINGLE_REG_SIZE            (8u)
#define W5200_SPI_USE_SECOND_DATAPATH        (W5200_SPI_DATA_WIDTH > W5200_SPI_SINGLE_REG_SIZE)

#define W5200_SPI_FIFO_SIZE                  (4u)
#define W5200_SPI_TX_SOFTWARE_BUF_ENABLED    ((0u != W5200_SPI_INTERNAL_TX_INT_ENABLED) && \
                                                     (W5200_SPI_TX_BUFFER_SIZE > W5200_SPI_FIFO_SIZE))

#define W5200_SPI_RX_SOFTWARE_BUF_ENABLED    ((0u != W5200_SPI_INTERNAL_RX_INT_ENABLED) && \
                                                     (W5200_SPI_RX_BUFFER_SIZE > W5200_SPI_FIFO_SIZE))


/***************************************
*        Data Struct Definition
***************************************/

/* Sleep Mode API Support */
typedef struct
{
    uint8 enableState;
    uint8 cntrPeriod;
} W5200_SPI_BACKUP_STRUCT;


/***************************************
*        Function Prototypes
***************************************/

void  W5200_SPI_Init(void)                           ;
void  W5200_SPI_Enable(void)                         ;
void  W5200_SPI_Start(void)                          ;
void  W5200_SPI_Stop(void)                           ;

void  W5200_SPI_EnableTxInt(void)                    ;
void  W5200_SPI_EnableRxInt(void)                    ;
void  W5200_SPI_DisableTxInt(void)                   ;
void  W5200_SPI_DisableRxInt(void)                   ;

void  W5200_SPI_Sleep(void)                          ;
void  W5200_SPI_Wakeup(void)                         ;
void  W5200_SPI_SaveConfig(void)                     ;
void  W5200_SPI_RestoreConfig(void)                  ;

void  W5200_SPI_SetTxInterruptMode(uint8 intSrc)     ;
void  W5200_SPI_SetRxInterruptMode(uint8 intSrc)     ;
uint8 W5200_SPI_ReadTxStatus(void)                   ;
uint8 W5200_SPI_ReadRxStatus(void)                   ;
void  W5200_SPI_WriteTxData(uint8 txData)  \
                                                            ;
uint8 W5200_SPI_ReadRxData(void) \
                                                            ;
uint8 W5200_SPI_GetRxBufferSize(void)                ;
uint8 W5200_SPI_GetTxBufferSize(void)                ;
void  W5200_SPI_ClearRxBuffer(void)                  ;
void  W5200_SPI_ClearTxBuffer(void)                  ;
void  W5200_SPI_ClearFIFO(void)                              ;
void  W5200_SPI_PutArray(const uint8 buffer[], uint8 byteCount) \
                                                            ;

#if(0u != W5200_SPI_BIDIRECTIONAL_MODE)
    void  W5200_SPI_TxEnable(void)                   ;
    void  W5200_SPI_TxDisable(void)                  ;
#endif /* (0u != W5200_SPI_BIDIRECTIONAL_MODE) */

CY_ISR_PROTO(W5200_SPI_TX_ISR);
CY_ISR_PROTO(W5200_SPI_RX_ISR);


/***************************************
*   Variable with external linkage
***************************************/

extern uint8 W5200_SPI_initVar;


/***************************************
*           API Constants
***************************************/

#define W5200_SPI_TX_ISR_NUMBER     ((uint8) (W5200_SPI_TxInternalInterrupt__INTC_NUMBER))
#define W5200_SPI_RX_ISR_NUMBER     ((uint8) (W5200_SPI_RxInternalInterrupt__INTC_NUMBER))

#define W5200_SPI_TX_ISR_PRIORITY   ((uint8) (W5200_SPI_TxInternalInterrupt__INTC_PRIOR_NUM))
#define W5200_SPI_RX_ISR_PRIORITY   ((uint8) (W5200_SPI_RxInternalInterrupt__INTC_PRIOR_NUM))


/***************************************
*    Initial Parameter Constants
***************************************/

#define W5200_SPI_INT_ON_SPI_DONE    ((uint8) (0u   << W5200_SPI_STS_SPI_DONE_SHIFT))
#define W5200_SPI_INT_ON_TX_EMPTY    ((uint8) (0u   << W5200_SPI_STS_TX_FIFO_EMPTY_SHIFT))
#define W5200_SPI_INT_ON_TX_NOT_FULL ((uint8) (0u << \
                                                                           W5200_SPI_STS_TX_FIFO_NOT_FULL_SHIFT))
#define W5200_SPI_INT_ON_BYTE_COMP   ((uint8) (0u  << W5200_SPI_STS_BYTE_COMPLETE_SHIFT))
#define W5200_SPI_INT_ON_SPI_IDLE    ((uint8) (0u   << W5200_SPI_STS_SPI_IDLE_SHIFT))

/* Disable TX_NOT_FULL if software buffer is used */
#define W5200_SPI_INT_ON_TX_NOT_FULL_DEF ((W5200_SPI_TX_SOFTWARE_BUF_ENABLED) ? \
                                                                        (0u) : (W5200_SPI_INT_ON_TX_NOT_FULL))

/* TX interrupt mask */
#define W5200_SPI_TX_INIT_INTERRUPTS_MASK    (W5200_SPI_INT_ON_SPI_DONE  | \
                                                     W5200_SPI_INT_ON_TX_EMPTY  | \
                                                     W5200_SPI_INT_ON_TX_NOT_FULL_DEF | \
                                                     W5200_SPI_INT_ON_BYTE_COMP | \
                                                     W5200_SPI_INT_ON_SPI_IDLE)

#define W5200_SPI_INT_ON_RX_FULL         ((uint8) (0u << \
                                                                          W5200_SPI_STS_RX_FIFO_FULL_SHIFT))
#define W5200_SPI_INT_ON_RX_NOT_EMPTY    ((uint8) (0u << \
                                                                          W5200_SPI_STS_RX_FIFO_NOT_EMPTY_SHIFT))
#define W5200_SPI_INT_ON_RX_OVER         ((uint8) (0u << \
                                                                          W5200_SPI_STS_RX_FIFO_OVERRUN_SHIFT))

/* RX interrupt mask */
#define W5200_SPI_RX_INIT_INTERRUPTS_MASK    (W5200_SPI_INT_ON_RX_FULL      | \
                                                     W5200_SPI_INT_ON_RX_NOT_EMPTY | \
                                                     W5200_SPI_INT_ON_RX_OVER)
/* Nubmer of bits to receive/transmit */
#define W5200_SPI_BITCTR_INIT            (((uint8) (W5200_SPI_DATA_WIDTH << 1u)) - 1u)


/***************************************
*             Registers
***************************************/
#if(CY_PSOC3 || CY_PSOC5)
    #define W5200_SPI_TXDATA_REG (* (reg8 *) \
                                                W5200_SPI_BSPIM_sR8_Dp_u0__F0_REG)
    #define W5200_SPI_TXDATA_PTR (  (reg8 *) \
                                                W5200_SPI_BSPIM_sR8_Dp_u0__F0_REG)
    #define W5200_SPI_RXDATA_REG (* (reg8 *) \
                                                W5200_SPI_BSPIM_sR8_Dp_u0__F1_REG)
    #define W5200_SPI_RXDATA_PTR (  (reg8 *) \
                                                W5200_SPI_BSPIM_sR8_Dp_u0__F1_REG)
#else   /* PSOC4 */
    #if(W5200_SPI_USE_SECOND_DATAPATH)
        #define W5200_SPI_TXDATA_REG (* (reg16 *) \
                                          W5200_SPI_BSPIM_sR8_Dp_u0__16BIT_F0_REG)
        #define W5200_SPI_TXDATA_PTR (  (reg16 *) \
                                          W5200_SPI_BSPIM_sR8_Dp_u0__16BIT_F0_REG)
        #define W5200_SPI_RXDATA_REG (* (reg16 *) \
                                          W5200_SPI_BSPIM_sR8_Dp_u0__16BIT_F1_REG)
        #define W5200_SPI_RXDATA_PTR (  (reg16 *) \
                                          W5200_SPI_BSPIM_sR8_Dp_u0__16BIT_F1_REG)
    #else
        #define W5200_SPI_TXDATA_REG (* (reg8 *) \
                                                W5200_SPI_BSPIM_sR8_Dp_u0__F0_REG)
        #define W5200_SPI_TXDATA_PTR (  (reg8 *) \
                                                W5200_SPI_BSPIM_sR8_Dp_u0__F0_REG)
        #define W5200_SPI_RXDATA_REG (* (reg8 *) \
                                                W5200_SPI_BSPIM_sR8_Dp_u0__F1_REG)
        #define W5200_SPI_RXDATA_PTR (  (reg8 *) \
                                                W5200_SPI_BSPIM_sR8_Dp_u0__F1_REG)
    #endif /* (W5200_SPI_USE_SECOND_DATAPATH) */
#endif     /* (CY_PSOC3 || CY_PSOC5) */

#define W5200_SPI_AUX_CONTROL_DP0_REG (* (reg8 *) \
                                        W5200_SPI_BSPIM_sR8_Dp_u0__DP_AUX_CTL_REG)
#define W5200_SPI_AUX_CONTROL_DP0_PTR (  (reg8 *) \
                                        W5200_SPI_BSPIM_sR8_Dp_u0__DP_AUX_CTL_REG)

#if(W5200_SPI_USE_SECOND_DATAPATH)
    #define W5200_SPI_AUX_CONTROL_DP1_REG  (* (reg8 *) \
                                        W5200_SPI_BSPIM_sR8_Dp_u1__DP_AUX_CTL_REG)
    #define W5200_SPI_AUX_CONTROL_DP1_PTR  (  (reg8 *) \
                                        W5200_SPI_BSPIM_sR8_Dp_u1__DP_AUX_CTL_REG)
#endif /* (W5200_SPI_USE_SECOND_DATAPATH) */

#define W5200_SPI_COUNTER_PERIOD_REG     (* (reg8 *) W5200_SPI_BSPIM_BitCounter__PERIOD_REG)
#define W5200_SPI_COUNTER_PERIOD_PTR     (  (reg8 *) W5200_SPI_BSPIM_BitCounter__PERIOD_REG)
#define W5200_SPI_COUNTER_CONTROL_REG    (* (reg8 *) W5200_SPI_BSPIM_BitCounter__CONTROL_AUX_CTL_REG)
#define W5200_SPI_COUNTER_CONTROL_PTR    (  (reg8 *) W5200_SPI_BSPIM_BitCounter__CONTROL_AUX_CTL_REG)

#define W5200_SPI_TX_STATUS_REG          (* (reg8 *) W5200_SPI_BSPIM_TxStsReg__STATUS_REG)
#define W5200_SPI_TX_STATUS_PTR          (  (reg8 *) W5200_SPI_BSPIM_TxStsReg__STATUS_REG)
#define W5200_SPI_RX_STATUS_REG          (* (reg8 *) W5200_SPI_BSPIM_RxStsReg__STATUS_REG)
#define W5200_SPI_RX_STATUS_PTR          (  (reg8 *) W5200_SPI_BSPIM_RxStsReg__STATUS_REG)

#define W5200_SPI_CONTROL_REG            (* (reg8 *) \
                                      W5200_SPI_BSPIM_BidirMode_CtrlReg__CONTROL_REG)
#define W5200_SPI_CONTROL_PTR            (  (reg8 *) \
                                      W5200_SPI_BSPIM_BidirMode_CtrlReg__CONTROL_REG)

#define W5200_SPI_TX_STATUS_MASK_REG     (* (reg8 *) W5200_SPI_BSPIM_TxStsReg__MASK_REG)
#define W5200_SPI_TX_STATUS_MASK_PTR     (  (reg8 *) W5200_SPI_BSPIM_TxStsReg__MASK_REG)
#define W5200_SPI_RX_STATUS_MASK_REG     (* (reg8 *) W5200_SPI_BSPIM_RxStsReg__MASK_REG)
#define W5200_SPI_RX_STATUS_MASK_PTR     (  (reg8 *) W5200_SPI_BSPIM_RxStsReg__MASK_REG)

#define W5200_SPI_TX_STATUS_ACTL_REG     (* (reg8 *) W5200_SPI_BSPIM_TxStsReg__STATUS_AUX_CTL_REG)
#define W5200_SPI_TX_STATUS_ACTL_PTR     (  (reg8 *) W5200_SPI_BSPIM_TxStsReg__STATUS_AUX_CTL_REG)
#define W5200_SPI_RX_STATUS_ACTL_REG     (* (reg8 *) W5200_SPI_BSPIM_RxStsReg__STATUS_AUX_CTL_REG)
#define W5200_SPI_RX_STATUS_ACTL_PTR     (  (reg8 *) W5200_SPI_BSPIM_RxStsReg__STATUS_AUX_CTL_REG)

#if(W5200_SPI_USE_SECOND_DATAPATH)
    #define W5200_SPI_AUX_CONTROLDP1     (W5200_SPI_AUX_CONTROL_DP1_REG)
#endif /* (W5200_SPI_USE_SECOND_DATAPATH) */


/***************************************
*       Register Constants
***************************************/

/* Status Register Definitions */
#define W5200_SPI_STS_SPI_DONE_SHIFT             (0x00u)
#define W5200_SPI_STS_TX_FIFO_EMPTY_SHIFT        (0x01u)
#define W5200_SPI_STS_TX_FIFO_NOT_FULL_SHIFT     (0x02u)
#define W5200_SPI_STS_BYTE_COMPLETE_SHIFT        (0x03u)
#define W5200_SPI_STS_SPI_IDLE_SHIFT             (0x04u)
#define W5200_SPI_STS_RX_FIFO_FULL_SHIFT         (0x04u)
#define W5200_SPI_STS_RX_FIFO_NOT_EMPTY_SHIFT    (0x05u)
#define W5200_SPI_STS_RX_FIFO_OVERRUN_SHIFT      (0x06u)

#define W5200_SPI_STS_SPI_DONE           ((uint8) (0x01u << W5200_SPI_STS_SPI_DONE_SHIFT))
#define W5200_SPI_STS_TX_FIFO_EMPTY      ((uint8) (0x01u << W5200_SPI_STS_TX_FIFO_EMPTY_SHIFT))
#define W5200_SPI_STS_TX_FIFO_NOT_FULL   ((uint8) (0x01u << W5200_SPI_STS_TX_FIFO_NOT_FULL_SHIFT))
#define W5200_SPI_STS_BYTE_COMPLETE      ((uint8) (0x01u << W5200_SPI_STS_BYTE_COMPLETE_SHIFT))
#define W5200_SPI_STS_SPI_IDLE           ((uint8) (0x01u << W5200_SPI_STS_SPI_IDLE_SHIFT))
#define W5200_SPI_STS_RX_FIFO_FULL       ((uint8) (0x01u << W5200_SPI_STS_RX_FIFO_FULL_SHIFT))
#define W5200_SPI_STS_RX_FIFO_NOT_EMPTY  ((uint8) (0x01u << W5200_SPI_STS_RX_FIFO_NOT_EMPTY_SHIFT))
#define W5200_SPI_STS_RX_FIFO_OVERRUN    ((uint8) (0x01u << W5200_SPI_STS_RX_FIFO_OVERRUN_SHIFT))

/* TX and RX masks for clear on read bits */
#define W5200_SPI_TX_STS_CLR_ON_RD_BYTES_MASK    (0x09u)
#define W5200_SPI_RX_STS_CLR_ON_RD_BYTES_MASK    (0x40u)

/* StatusI Register Interrupt Enable Control Bits */
/* As defined by the Register map for the AUX Control Register */
#define W5200_SPI_INT_ENABLE     (0x10u) /* Enable interrupt from statusi */
#define W5200_SPI_TX_FIFO_CLR    (0x01u) /* F0 - TX FIFO */
#define W5200_SPI_RX_FIFO_CLR    (0x02u) /* F1 - RX FIFO */
#define W5200_SPI_FIFO_CLR       (W5200_SPI_TX_FIFO_CLR | W5200_SPI_RX_FIFO_CLR)

/* Bit Counter (7-bit) Control Register Bit Definitions */
/* As defined by the Register map for the AUX Control Register */
#define W5200_SPI_CNTR_ENABLE    (0x20u) /* Enable CNT7 */

/* Bi-Directional mode control bit */
#define W5200_SPI_CTRL_TX_SIGNAL_EN  (0x01u)

/* Datapath Auxillary Control Register definitions */
#define W5200_SPI_AUX_CTRL_FIFO0_CLR         (0x01u)
#define W5200_SPI_AUX_CTRL_FIFO1_CLR         (0x02u)
#define W5200_SPI_AUX_CTRL_FIFO0_LVL         (0x04u)
#define W5200_SPI_AUX_CTRL_FIFO1_LVL         (0x08u)
#define W5200_SPI_STATUS_ACTL_INT_EN_MASK    (0x10u)

/* Component disabled */
#define W5200_SPI_DISABLED   (0u)


/***************************************
*       Macros
***************************************/

/* Returns true if componentn enabled */
#define W5200_SPI_IS_ENABLED (0u != (W5200_SPI_TX_STATUS_ACTL_REG & W5200_SPI_INT_ENABLE))

/* Retuns TX status register */
#define W5200_SPI_GET_STATUS_TX(swTxSts) ( (uint8)(W5200_SPI_TX_STATUS_REG | \
                                                          ((swTxSts) & W5200_SPI_TX_STS_CLR_ON_RD_BYTES_MASK)) )
/* Retuns RX status register */
#define W5200_SPI_GET_STATUS_RX(swRxSts) ( (uint8)(W5200_SPI_RX_STATUS_REG | \
                                                          ((swRxSts) & W5200_SPI_RX_STS_CLR_ON_RD_BYTES_MASK)) )


/***************************************
* The following code is DEPRECATED and 
* should not be used in new projects.
***************************************/

#define W5200_SPI_WriteByte   W5200_SPI_WriteTxData
#define W5200_SPI_ReadByte    W5200_SPI_ReadRxData
void  W5200_SPI_SetInterruptMode(uint8 intSrc)       ;
uint8 W5200_SPI_ReadStatus(void)                     ;
void  W5200_SPI_EnableInt(void)                      ;
void  W5200_SPI_DisableInt(void)                     ;

#define W5200_SPI_TXDATA                 (W5200_SPI_TXDATA_REG)
#define W5200_SPI_RXDATA                 (W5200_SPI_RXDATA_REG)
#define W5200_SPI_AUX_CONTROLDP0         (W5200_SPI_AUX_CONTROL_DP0_REG)
#define W5200_SPI_TXBUFFERREAD           (W5200_SPI_txBufferRead)
#define W5200_SPI_TXBUFFERWRITE          (W5200_SPI_txBufferWrite)
#define W5200_SPI_RXBUFFERREAD           (W5200_SPI_rxBufferRead)
#define W5200_SPI_RXBUFFERWRITE          (W5200_SPI_rxBufferWrite)

#define W5200_SPI_COUNTER_PERIOD         (W5200_SPI_COUNTER_PERIOD_REG)
#define W5200_SPI_COUNTER_CONTROL        (W5200_SPI_COUNTER_CONTROL_REG)
#define W5200_SPI_STATUS                 (W5200_SPI_TX_STATUS_REG)
#define W5200_SPI_CONTROL                (W5200_SPI_CONTROL_REG)
#define W5200_SPI_STATUS_MASK            (W5200_SPI_TX_STATUS_MASK_REG)
#define W5200_SPI_STATUS_ACTL            (W5200_SPI_TX_STATUS_ACTL_REG)

#define W5200_SPI_INIT_INTERRUPTS_MASK  (W5200_SPI_INT_ON_SPI_DONE     | \
                                                W5200_SPI_INT_ON_TX_EMPTY     | \
                                                W5200_SPI_INT_ON_TX_NOT_FULL_DEF  | \
                                                W5200_SPI_INT_ON_RX_FULL      | \
                                                W5200_SPI_INT_ON_RX_NOT_EMPTY | \
                                                W5200_SPI_INT_ON_RX_OVER      | \
                                                W5200_SPI_INT_ON_BYTE_COMP)
                                                
#define W5200_SPI_DataWidth                  (W5200_SPI_DATA_WIDTH)
#define W5200_SPI_InternalClockUsed          (W5200_SPI_INTERNAL_CLOCK)
#define W5200_SPI_InternalTxInterruptEnabled (W5200_SPI_INTERNAL_TX_INT_ENABLED)
#define W5200_SPI_InternalRxInterruptEnabled (W5200_SPI_INTERNAL_RX_INT_ENABLED)
#define W5200_SPI_ModeUseZero                (W5200_SPI_MODE_USE_ZERO)
#define W5200_SPI_BidirectionalMode          (W5200_SPI_BIDIRECTIONAL_MODE)
#define W5200_SPI_Mode                       (W5200_SPI_MODE)
#define W5200_SPI_DATAWIDHT                  (W5200_SPI_DATA_WIDTH)
#define W5200_SPI_InternalInterruptEnabled   (0u)

#define W5200_SPI_TXBUFFERSIZE   (W5200_SPI_TX_BUFFER_SIZE)
#define W5200_SPI_RXBUFFERSIZE   (W5200_SPI_RX_BUFFER_SIZE)

#define W5200_SPI_TXBUFFER       W5200_SPI_txBuffer
#define W5200_SPI_RXBUFFER       W5200_SPI_rxBuffer

#endif /* (CY_SPIM_W5200_SPI_H) */


/* [] END OF FILE */
