/*******************************************************************************
* File Name: W5200_SPI_INT.c
* Version 2.50
*
* Description:
*  This file provides all Interrupt Service Routine (ISR) for the SPI Master
*  component.
*
* Note:
*  None.
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "W5200_SPI_PVT.h"
#include "cyapicallbacks.h"

/* User code required at start of ISR */
/* `#START W5200_SPI_ISR_START_DEF` */

/* `#END` */


/*******************************************************************************
* Function Name: W5200_SPI_TX_ISR
********************************************************************************
*
* Summary:
*  Interrupt Service Routine for TX portion of the SPI Master.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  W5200_SPI_txBufferWrite - used for the account of the bytes which
*  have been written down in the TX software buffer.
*  W5200_SPI_txBufferRead - used for the account of the bytes which
*  have been read from the TX software buffer, modified when exist data to
*  sending and FIFO Not Full.
*  W5200_SPI_txBuffer[W5200_SPI_TX_BUFFER_SIZE] - used to store
*  data to sending.
*  All described above Global variables are used when Software Buffer is used.
*
*******************************************************************************/
CY_ISR(W5200_SPI_TX_ISR)
{
    #if(W5200_SPI_TX_SOFTWARE_BUF_ENABLED)
        uint8 tmpStatus;
    #endif /* (W5200_SPI_TX_SOFTWARE_BUF_ENABLED) */

    #ifdef W5200_SPI_TX_ISR_ENTRY_CALLBACK
        W5200_SPI_TX_ISR_EntryCallback();
    #endif /* W5200_SPI_TX_ISR_ENTRY_CALLBACK */

    /* User code required at start of ISR */
    /* `#START W5200_SPI_TX_ISR_START` */

    /* `#END` */
    
    #if(W5200_SPI_TX_SOFTWARE_BUF_ENABLED)
        /* Check if TX data buffer is not empty and there is space in TX FIFO */
        while(W5200_SPI_txBufferRead != W5200_SPI_txBufferWrite)
        {
            tmpStatus = W5200_SPI_GET_STATUS_TX(W5200_SPI_swStatusTx);
            W5200_SPI_swStatusTx = tmpStatus;

            if(0u != (W5200_SPI_swStatusTx & W5200_SPI_STS_TX_FIFO_NOT_FULL))
            {
                if(0u == W5200_SPI_txBufferFull)
                {
                   W5200_SPI_txBufferRead++;

                    if(W5200_SPI_txBufferRead >= W5200_SPI_TX_BUFFER_SIZE)
                    {
                        W5200_SPI_txBufferRead = 0u;
                    }
                }
                else
                {
                    W5200_SPI_txBufferFull = 0u;
                }

                /* Put data element into the TX FIFO */
                CY_SET_REG8(W5200_SPI_TXDATA_PTR, 
                                             W5200_SPI_txBuffer[W5200_SPI_txBufferRead]);
            }
            else
            {
                break;
            }
        }

        if(W5200_SPI_txBufferRead == W5200_SPI_txBufferWrite)
        {
            /* TX Buffer is EMPTY: disable interrupt on TX NOT FULL */
            W5200_SPI_TX_STATUS_MASK_REG &= ((uint8) ~W5200_SPI_STS_TX_FIFO_NOT_FULL);
        }

    #endif /* (W5200_SPI_TX_SOFTWARE_BUF_ENABLED) */

    /* User code required at end of ISR (Optional) */
    /* `#START W5200_SPI_TX_ISR_END` */

    /* `#END` */
    
    #ifdef W5200_SPI_TX_ISR_EXIT_CALLBACK
        W5200_SPI_TX_ISR_ExitCallback();
    #endif /* W5200_SPI_TX_ISR_EXIT_CALLBACK */
}


/*******************************************************************************
* Function Name: W5200_SPI_RX_ISR
********************************************************************************
*
* Summary:
*  Interrupt Service Routine for RX portion of the SPI Master.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global variables:
*  W5200_SPI_rxBufferWrite - used for the account of the bytes which
*  have been written down in the RX software buffer modified when FIFO contains
*  new data.
*  W5200_SPI_rxBufferRead - used for the account of the bytes which
*  have been read from the RX software buffer, modified when overflow occurred.
*  W5200_SPI_rxBuffer[W5200_SPI_RX_BUFFER_SIZE] - used to store
*  received data, modified when FIFO contains new data.
*  All described above Global variables are used when Software Buffer is used.
*
*******************************************************************************/
CY_ISR(W5200_SPI_RX_ISR)
{
    #if(W5200_SPI_RX_SOFTWARE_BUF_ENABLED)
        uint8 tmpStatus;
        uint8 rxData;
    #endif /* (W5200_SPI_RX_SOFTWARE_BUF_ENABLED) */

    #ifdef W5200_SPI_RX_ISR_ENTRY_CALLBACK
        W5200_SPI_RX_ISR_EntryCallback();
    #endif /* W5200_SPI_RX_ISR_ENTRY_CALLBACK */

    /* User code required at start of ISR */
    /* `#START W5200_SPI_RX_ISR_START` */

    /* `#END` */
    
    #if(W5200_SPI_RX_SOFTWARE_BUF_ENABLED)

        tmpStatus = W5200_SPI_GET_STATUS_RX(W5200_SPI_swStatusRx);
        W5200_SPI_swStatusRx = tmpStatus;

        /* Check if RX data FIFO has some data to be moved into the RX Buffer */
        while(0u != (W5200_SPI_swStatusRx & W5200_SPI_STS_RX_FIFO_NOT_EMPTY))
        {
            rxData = CY_GET_REG8(W5200_SPI_RXDATA_PTR);

            /* Set next pointer. */
            W5200_SPI_rxBufferWrite++;
            if(W5200_SPI_rxBufferWrite >= W5200_SPI_RX_BUFFER_SIZE)
            {
                W5200_SPI_rxBufferWrite = 0u;
            }

            if(W5200_SPI_rxBufferWrite == W5200_SPI_rxBufferRead)
            {
                W5200_SPI_rxBufferRead++;
                if(W5200_SPI_rxBufferRead >= W5200_SPI_RX_BUFFER_SIZE)
                {
                    W5200_SPI_rxBufferRead = 0u;
                }

                W5200_SPI_rxBufferFull = 1u;
            }

            /* Move data from the FIFO to the Buffer */
            W5200_SPI_rxBuffer[W5200_SPI_rxBufferWrite] = rxData;

            tmpStatus = W5200_SPI_GET_STATUS_RX(W5200_SPI_swStatusRx);
            W5200_SPI_swStatusRx = tmpStatus;
        }

    #endif /* (W5200_SPI_RX_SOFTWARE_BUF_ENABLED) */

    /* User code required at end of ISR (Optional) */
    /* `#START W5200_SPI_RX_ISR_END` */

    /* `#END` */
    
    #ifdef W5200_SPI_RX_ISR_EXIT_CALLBACK
        W5200_SPI_RX_ISR_ExitCallback();
    #endif /* W5200_SPI_RX_ISR_EXIT_CALLBACK */
}

/* [] END OF FILE */
